//////////////////////////////////////////////////////////////////////
//
//  NtRegEdit.h: main header file for the NtRegEdit application
//
//////////////////////////////////////////////////////////////////////
//
// File           : NtRegEdit.cpp
// Version        : 0.0.2.734
// Function       : Source file of the Native Registry Editor.
// Copyright      :  2006 Daniel Madden
// Author         : Daniel Madden
// Notes          : The program was created by using the Native 
//					Registry APIs (found in the book by Gary 
//					Nebbett - see below) and the "Look & Feel" of 
//					the Microsoft "RegEdit" program. 
//
//					There is a Class I wrote that this app uses and
//					can be found here:
//					http://www.codeproject.com/system/NtRegistry.asp.
//
// Modifications  :
//
// Rev 0.0.2.857	August 23, 2006	Daniel Madden
//	- Added Permissions dialog
//
// Rev 0.0.2.734	August 10, 2006	Daniel Madden
//	- Added names to the About dialog (source owners)
//	- Added bold options to the TreeCtrl
//	- Added Application Settings Dialog boxes
//		- COptGenDlg() - General
//		- COptListDlg() - List Control
//		- COptTreeDlg() - Tree Control
//	- Added coloring to the CNtRegEditTreeView class (XListCtrl already had it)
//	- Added Label editing for the CTreeCtrl
//	- Added the the begining of the NEW "NtRegEdit" format.
//	- Added Virtual Keys:
//		- VK_UP (Up to next "Visible" item (displays valuses)
//		- VK_DOWN (Down to next "Visible" item (displays valuses)
//		- VK_DELETE (will prompt user for recursive if subkey exists)
//		- VK_ENTER (Opens a subkey if it exists, else no effect)
//	- Fixed "OnKeysRefresh()" function (I think).
//
// Rev 0.0.1.689	July 16, 2006	Daniel Madden
//	- Added Key path in the Statusbar.
//
// Rev 0.0.0.1	June 3, 2004	Daniel Madden
//	- Initial creation
// 
//
// Copyright  2006 Daniel Madden
//
// This grants you ("Licensee") a non-exclusive, royalty free, 
// licence to use, modify and redistribute this software in source and binary 
// code form, provided that i) this copyright notice and licence appear on all 
// copies of the software; and ii) Licensee does not utilize the software in a 
// manner which is disparaging to Daniel Madden.
//
// This software is provided "AS IS," without a warranty of any kind. ALL
// EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING 
// ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
// OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. JETBYTE LIMITED AND ITS LICENSORS 
// SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF 
// USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO 
// EVENT WILL JETBYTE LIMITED BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, 
// OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE 
// DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING 
// OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF JETBYTE LIMITED 
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//////////////////////////////////////////////////////////////////////

#pragma once

#ifndef __AFXWIN_H__
	#error "include 'stdafx.h' before including this file for PCH"
#endif

#include "resource.h"       // main symbols

/*
Value Type					API Code#	Value Description 
==========================  =========   ==========================================
REG_ERROR						-1		Error request value 
REG_NONE						0		Null value 
REG_SZ							1		Null terminated Unicode fixed string value 
REG_EXPAND_SZ					2		Null terminated unexpanded Unicode/ANSI environment string value 
REG_BINARY						3		Binary value of any form/length 
REG_DWORD						4		32-bit numerical value 
REG_DWORD_LITTLE_ENDIAN			4		32-bit numerical value 
REG_DWORD_BIG_ENDIAN			5		32-bit reversed numerical value 
REG_LINK						6		Symbolic Unicode link string value 
REG_MULTI_SZ					7		Array of multiple Unicode strings separated/ended by null characters 
REG_RESOURCE_LIST				8		Device driver list of hardware resources in Resource Map tree 
REG_FULL_RESOURCE_DESCRIPTOR	9		List of hardware resources in Description tree 
REG_RESOURCE_REQUIREMENTS_LIST	10		Device driver list of hardware resource requirements in Resource Map tree 
REG_QWORD						11		64-bit numerical value 
REG_QWORD_LITTLE_ENDIAN			11		64-bit numerical value 
*/

#include "MainFrm.h"
#include "NtRegEditTreeView.h"
#include "NtRegEditView.h"
#include "BinaryView.h"
#include "DWordView.h"
#include "MStringView.h"
#include "StringView.h"
#include "FindSearchView.h"

#define	MAJ_VER	0
#define	MIN_VER	0
#define	BLD_VER	2
#define	PRI_VER	857

// CNtRegEditApp:
// See NtRegEdit.cpp for the implementation of this class
class CNtRegEditApp : public CWinApp
{
public:
	CNtRegEditApp();

// Attributes
public:

	CMainFrame			*m_clsMainFrm;
	TVisualFramework	*m_clsFramework;
	CNtRegEditTreeView	*m_clsTV;
	CNtRegEditView		*m_clsLV;
	CBinaryView			*m_clsBin;
	CDWordView			*m_clsDWord;
	CMStringView		*m_clsMString;
	CStringView			*m_clsString;
	CFindSearchView		*m_clsFS;

	CDWordArray			m_dwaTVisualObject;

	BOOL				m_bWindowOnTop;
	BOOL				m_bSaveWindowPos;
	BOOL				m_bCheckOSType;
	BOOL				m_bCheckIsUserAdmin;


// Operations
public:

	int		Hex2Dec(CString csHex);
	BOOL	DoesFileExist(CString csFilename);
	BYTE	GetAttributes(CString csFilename);
	void	SetAttributes(CString csFilename, BYTE btAttrib);

	CString GetLocalMachineName(void);
	//BOOL	ConnectNtRegistry(CString lpMachineName, HKEY hKey);

// Overrides
public:
	virtual BOOL InitInstance();

// Implementation
	afx_msg void OnAppAbout();
	DECLARE_MESSAGE_MAP()
	afx_msg BOOL OnOpenRecentFile(UINT nID);
};

extern CNtRegEditApp theApp;

