// OptGeneralDlg.cpp : implementation file
//

#include "stdafx.h"
#include "NtRegEdit.h"

#include "SelColorDlg.h"
  #ifndef __FOLDERDLG_H__
	#include "FolderDlg.h"
#endif

#ifndef __FILTERED_FOLDERDLG_H__
	#include "FilteredFolderDlg.h"
#endif

#include "NtRegistry.h"

#include "OptGeneralDlg.h"

// COptGeneralDlg dialog
IMPLEMENT_DYNAMIC(COptGeneralDlg, CSAPrefsSubDlg)

COptGeneralDlg::COptGeneralDlg(CWnd* pParent /*=NULL*/)
	: CSAPrefsSubDlg(COptGeneralDlg::IDD, pParent)
{
	m_bItemsModified = FALSE;
}

COptGeneralDlg::~COptGeneralDlg()
{
}

void COptGeneralDlg::DoDataExchange(CDataExchange* pDX)
{
	CSAPrefsSubDlg::DoDataExchange(pDX);
	DDX_Control(pDX, BTN_APPLY, m_btnApply);
	DDX_Control(pDX, BTN_BROWSE_PATH, m_btnPath);
	DDX_Control(pDX, BTN_BROWSE_LOGS, m_btnLogs);
	DDX_Control(pDX, CHK_IS_USER_ADMIN, m_chkIsUserAdmin);
	DDX_Control(pDX, CHK_OS_TYPE, m_chkOSType);
	DDX_Control(pDX, CHK_SAVE_WND_POSITION, m_chkSaveWndPos);
	DDX_Control(pDX, CHK_WND_ON_TOP, m_chkWndOnTop);
	DDX_Control(pDX, EDT_MAX_LOG_SIZE, m_edtLogSize);
	DDX_Control(pDX, EDT_LOGS_DIR, m_edtLogs);
	DDX_Control(pDX, EDT_NTREGEDIT_DIR, m_edtPath);
}


BEGIN_MESSAGE_MAP(COptGeneralDlg, CSAPrefsSubDlg)
	ON_BN_CLICKED(BTN_BROWSE_LOGS, OnBrowseLogs)
	ON_BN_CLICKED(BTN_BROWSE_PATH, OnBrowsePath)
	ON_BN_CLICKED(BTN_TEXT_COLOR, OnTextColor)
	ON_BN_CLICKED(BTN_BG_COLOR, OnBgColor)
	ON_BN_CLICKED(BTN_SHADOW_COLOR, OnShadowColor)
	ON_BN_CLICKED(BTN_APPLY, OnApply)
	ON_BN_CLICKED(CHK_IS_USER_ADMIN, OnBnClickedIsUserAdmin)
	ON_BN_CLICKED(CHK_OS_TYPE, OnBnClickedOsType)
	ON_BN_CLICKED(CHK_SAVE_WND_POSITION, OnBnClickedSaveWndPosition)
	ON_BN_CLICKED(CHK_WND_ON_TOP, OnBnClickedWndOnTop)
	ON_EN_CHANGE(EDT_MAX_LOG_SIZE, OnEnChangeMaxLogSize)
END_MESSAGE_MAP()


// COptGeneralDlg message handlers
BOOL COptGeneralDlg::OnInitDialog() 
{
	CSAPrefsSubDlg::OnInitDialog();

	m_bColorModified = FALSE;

	m_crText = RGB(255,255,255);
	m_crBG = RGB(240,240,240);
	m_crShadow = RGB(0,0,0);
	
	CNtRegistry reg;
	{
		if (reg.SetKey(HKEY_CURRENT_USER,"Software\\NtRegEdit\\Settings",TRUE,TRUE))
		{
			m_csAppDir = reg.ReadString("Path","ERR");
			m_edtPath.SetWindowText(m_csAppDir);

			m_csTempDir = reg.ReadString("Temp","ERR");

			m_csAppLogDir = reg.ReadString("Logs","ERR");
			m_edtLogs.SetWindowText(m_csAppLogDir);

			m_dwMaxLogFileSize = reg.ReadDword("MaxLogFileSize",250000);
			CString csText;
			csText.Format("%d",m_dwMaxLogFileSize);
			m_edtLogSize.SetWindowText(csText);

			m_bWindowOnTop = reg.ReadBool(_T("AlwaysOnTop"),FALSE);
			m_chkWndOnTop.SetCheck(m_bWindowOnTop);

			m_bSaveWindowPos = reg.ReadBool(_T("SaveWindowPos"),TRUE);
			m_chkSaveWndPos.SetCheck(m_bSaveWindowPos);

			theApp.m_bCheckOSType = reg.ReadBool(_T("CheckOSType"),theApp.m_bCheckOSType);
			m_chkOSType.SetCheck(theApp.m_bCheckOSType);

			theApp.m_bCheckIsUserAdmin = reg.ReadBool(_T("CheckIsUserAdmin"),theApp.m_bCheckIsUserAdmin);
			m_chkIsUserAdmin.SetCheck(theApp.m_bCheckIsUserAdmin);

			m_crText = reg.ReadColor("TextColor",RGB(255,255,255));
			m_crBG = reg.ReadColor("BGColor",RGB(240,240,240));
			m_crShadow = reg.ReadColor("ShadowColor",RGB(0,0,0));
		}
	}

	m_bItemsModified = FALSE;
	m_btnApply.EnableWindow(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void COptGeneralDlg::OnBrowsePath() 
{
	LPCTSTR lpszTitle = _T( "Select the NtRegEdit Path Folder" );
	UINT	uFlags	  = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;

	CFolderDialog* pFolderDlg = NULL;
	pFolderDlg = (CFolderDialog*)new CFilteredFolderDialog( lpszTitle, m_csAppDir, this, uFlags );
	
	((CFilteredFolderDialog*)pFolderDlg)->SetFilter("NtRegEdit.exe;");
	
	ASSERT( pFolderDlg != NULL );
	if( pFolderDlg != NULL )
	{
		pFolderDlg->SetRootFolder( _T("C:") );
		if( pFolderDlg->DoModal() == IDOK )
		{
			m_csAppDir = pFolderDlg->GetFolderPath();
			SAFE_DELETE( pFolderDlg );		
		}
		else {
			SAFE_DELETE( pFolderDlg );		
			return; // open cancelled
		}
	}

	m_edtPath.SetWindowText(m_csAppDir);
	m_bItemsModified = TRUE;
	m_btnApply.EnableWindow(TRUE);
}

void COptGeneralDlg::OnBrowseLogs() 
{
	LPCTSTR lpszTitle = _T( "Select the NtRegEdit Log Folder" );
	UINT	uFlags	  = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;

	CFolderDialog* pFolderDlg = NULL;
	pFolderDlg = (CFolderDialog*)new CFilteredFolderDialog( lpszTitle, m_csAppLogDir, this, uFlags );
	
	((CFilteredFolderDialog*)pFolderDlg)->SetFilter("NtRegEdit.log;");
	
	ASSERT( pFolderDlg != NULL );
	if( pFolderDlg != NULL )
	{
		pFolderDlg->SetRootFolder( _T("C:") );
		if( pFolderDlg->DoModal() == IDOK )
		{
			m_csAppLogDir = pFolderDlg->GetFolderPath();
			SAFE_DELETE( pFolderDlg );		
		}
		else {
			SAFE_DELETE( pFolderDlg );		
			return; // open cancelled
		}
	}

	m_edtLogs.SetWindowText(m_csAppLogDir);
	m_bItemsModified = TRUE;
	m_btnApply.EnableWindow(TRUE);
}

void COptGeneralDlg::OnTextColor() 
{
	CNtRegistry reg;
	if (reg.SetRootKey(HKEY_CURRENT_USER))
	{
		if (reg.SetKey("Software\\NtRegEdit\\Settings",TRUE,TRUE))
		{
			m_crText = reg.ReadColor("TextColor",RGB(255,255,255));

			// create an instance of the color selector
			CSelColorDlg dlgSelColor(GetRValue(m_crText),GetGValue(m_crText),GetBValue(m_crText));
			// display the color changing dialog box
			if(dlgSelColor.DoModal() == IDOK)
			{
				m_bColorModified = TRUE;
				m_crText = dlgSelColor.GetColor();
				m_bItemsModified = TRUE;
				m_btnApply.EnableWindow(TRUE);
			}
		}
	}
}

void COptGeneralDlg::OnBgColor() 
{
	CNtRegistry reg;
	if (reg.SetRootKey(HKEY_CURRENT_USER))
	{
		if (reg.SetKey("Software\\NtRegEdit\\Settings",TRUE,TRUE))
		{
			m_crBG = reg.ReadColor("BGColor",RGB(240,240,240));

			// create an instance of the color selector
			CSelColorDlg dlgSelColor(GetRValue(m_crBG),GetGValue(m_crBG),GetBValue(m_crBG));
			// display the color changing dialog box
			if(dlgSelColor.DoModal() == IDOK)
			{
				m_bColorModified = TRUE;
				m_crBG = dlgSelColor.GetColor();
				m_bItemsModified = TRUE;
				m_btnApply.EnableWindow(TRUE);
			}
		}
	}
}

void COptGeneralDlg::OnShadowColor() 
{
	CNtRegistry reg;
	if (reg.SetRootKey(HKEY_CURRENT_USER))
	{
		if (reg.SetKey("Software\\NtRegEdit\\Settings",TRUE,TRUE))
		{
			m_crShadow = reg.ReadColor("ShadowColor",RGB(0,0,0));

			// create an instance of the color selector
			CSelColorDlg dlgSelColor(GetRValue(m_crShadow),GetGValue(m_crShadow),GetBValue(m_crShadow));
			// display the color changing dialog box
			if(dlgSelColor.DoModal() == IDOK)
			{
				m_bColorModified = TRUE;
				m_crShadow = dlgSelColor.GetColor();
				m_bItemsModified = TRUE;
				m_btnApply.EnableWindow(TRUE);
			}
		}
	}
}

void COptGeneralDlg::OnApply() 
{
	CNtRegistry reg;
	if (reg.SetKey(HKEY_CURRENT_USER,"Software\\NtRegEdit\\Settings",TRUE,TRUE))
	{
		reg.WriteColor("TextColor",m_crText);
		reg.WriteColor("BGColor",m_crBG);
		reg.WriteColor("ShadowColor",m_crShadow);

		if (m_bColorModified) {
			m_bColorModified = FALSE;
			MessageBox("These changes will not take effect until you re-open the dialog.");
		}

		reg.WriteString("Path",m_csAppDir);
		reg.WriteString("Logs",m_csAppLogDir);
		reg.WriteString("Temp",m_csTempDir);
		reg.WriteDword("MaxLogFileSize",m_dwMaxLogFileSize);
		reg.WriteBool(_T("AlwaysOnTop"),m_bWindowOnTop);
		reg.WriteBool(_T("SaveWindowPos"),m_bSaveWindowPos);
		reg.WriteBool(_T("CheckOSType"),theApp.m_bCheckOSType);
		reg.WriteBool(_T("CheckIsUserAdmin"),theApp.m_bCheckIsUserAdmin);

		m_bItemsModified = FALSE;
		m_btnApply.EnableWindow(FALSE);
	}

	const CWnd* pWnd = &CWnd::wndNoTopMost;
	if (m_bWindowOnTop) { pWnd = &CWnd::wndTopMost; }
	AfxGetMainWnd()->SetWindowPos(pWnd,0,0,0,0,SWP_NOSIZE|SWP_NOMOVE);

}

void COptGeneralDlg::OnBnClickedIsUserAdmin()
{
	theApp.m_bCheckIsUserAdmin = m_chkIsUserAdmin.GetCheck();
	m_bItemsModified = TRUE;
	m_btnApply.EnableWindow(TRUE);
}

void COptGeneralDlg::OnBnClickedOsType()
{
	theApp.m_bCheckOSType = m_chkOSType.GetCheck();
	m_bItemsModified = TRUE;
	m_btnApply.EnableWindow(TRUE);
}

void COptGeneralDlg::OnBnClickedSaveWndPosition()
{
	m_bSaveWindowPos = m_chkSaveWndPos.GetCheck();
	m_bItemsModified = TRUE;
	m_btnApply.EnableWindow(TRUE);
}

void COptGeneralDlg::OnBnClickedWndOnTop()
{
	m_bWindowOnTop = (BOOL)m_chkWndOnTop.GetCheck();
	m_bItemsModified = TRUE;
	m_btnApply.EnableWindow(TRUE);
}

void COptGeneralDlg::OnEnChangeMaxLogSize()
{
	CString csText;
	csText.Format("%d",m_dwMaxLogFileSize);
	m_edtLogSize.GetWindowText(csText);
	m_dwMaxLogFileSize = atoi(csText);
	m_bItemsModified = TRUE;
	m_btnApply.EnableWindow(TRUE);
}

void COptGeneralDlg::OnOK()
{
	if (m_bItemsModified) {
		if (MessageBox("Some items have been modified! Do you want to save the changes?",
			           "GENERAL - Modifications ",
					   MB_YESNO|MB_ICONQUESTION) == IDYES) {
			COptGeneralDlg::OnApply();
		}
	}
	CSAPrefsSubDlg::OnOK();
}
