/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

**********************************************************************/

#pragma once

#include "SAPrefsStatic.h"
#include "SAPrefsSubDlg.h"

#include <afxtempl.h>

class pageStruct
{
   public:
   CSAPrefsSubDlg *pDlg;
   UINT id;
   CSAPrefsSubDlg *pDlgParent;
   CString csCaption;
};

#define WM_CHANGE_PAGE (WM_APP + 100)
#define WM_SET_FOCUS_WND (WM_APP + 101)

// CSAPrefsDialog dialog
class CSAPrefsDialog : public CDialog
{
	DECLARE_DYNAMIC(CSAPrefsDialog)

public:
	CSAPrefsDialog(CWnd* pParent = NULL);   // standard constructor
	virtual ~CSAPrefsDialog();

// Dialog Data
	enum { IDD = IDD_SAPREFS };
	CStatic	m_boundingFrame;

	// dialog title
	void SetTitle(CString t)   {m_csTitle = t;}
	
	// used in the pretty shaded static control
	void SetConstantText(CString t)   {m_csConstantText = t;}

	// used in the pretty shaded static control
	void SetConstantTextColor(COLORREF cr)   {m_crTextColor = cr;}

	// used in the pretty shaded static control
	void SetConstantBGColor(COLORREF cr)   {m_crBGColor = cr;}

	// used in the pretty shaded static control
	void SetConstantShadowColor(COLORREF cr)   {m_crShadowColor = cr;}

	// if you set this, UpdateData will be called on the current page before
	// switching to the new page. if UpdateData fails, the page is not changed.
	void SetValidateBeforeChangingPages(bool b)	{m_bValidateBeforeChangingPage = b;}

	// add a page (page, page title, optional parent)
	bool AddPage(CSAPrefsSubDlg &page, const char *pCaption, CSAPrefsSubDlg *pDlgParent = NULL);
	
	// show a page
	bool ShowPage(int iPage);
	bool ShowPage(CSAPrefsSubDlg * pPage);
	
	// end the dialog with a special return code
	void EndSpecial(UINT res, bool bOk = true);
	
	// set the first page
	void SetStartPage(CSAPrefsSubDlg *pPage = NULL) {m_pStartPage = pPage;}
	
public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);

	bool EndOK();

	// if you don't like this, you can replace it with a static
	CSAPrefsStatic	m_captionBar;
	CTreeCtrl		m_pageTree;

	// check to see if this dlg has already been added to the tree
	HTREEITEM FindHTREEItemForDlg(CSAPrefsSubDlg *pParent);

	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	virtual BOOL OnInitDialog();
	virtual void OnOK();
	virtual void OnCancel();
	afx_msg void OnSelchangedPageTree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnGetdispinfoPageTree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnPhelp();
	afx_msg void OnSelchangingPageTree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnPaint();
	afx_msg long OnChangePage(UINT, LONG);
	afx_msg long OnSetFocusWnd(UINT, LONG);
	DECLARE_MESSAGE_MAP()

	CPtrArray   m_pages;
	int         m_iCurPage;
	CRect       m_frameRect;
	CString     m_csTitle, m_csConstantText;
	COLORREF	m_crTextColor, m_crBGColor, m_crShadowColor;

	bool m_bValidateBeforeChangingPage;

	CSAPrefsSubDlg	*m_pStartPage;

	// store info about *pDlgs that have been added to 
	// the tree - used for quick lookup of parent nodes
	// DWORDs are used because HTREEITEMs can't be... blame Microsoft
	CMap< CSAPrefsSubDlg *, CSAPrefsSubDlg *, DWORD, DWORD&  > m_dlgMap;

};
