/////////////////////////////////////////////////////////////
//	
//	Author:		Sami (M.ALSAMSAM), ittiger@ittiger.net
//
//	Filename:	TreeCtrlEx.cpp
//
//	http	 :	www.ittiger.net
//
//////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NtRegEdit.h"
#include "TreeCtrlEx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//////////////////////////////////
//IMPLEMENT_DYNAMIC(CTreeCtrlEx, CTreeCtrl)
BEGIN_MESSAGE_MAP(CTreeCtrlEx, CTreeCtrl)
	ON_WM_PAINT()
	ON_WM_CREATE()
END_MESSAGE_MAP()

//////////////////////////////////
CTreeCtrlEx::CTreeCtrlEx()
{
}

CTreeCtrlEx::~CTreeCtrlEx()
{
}

int CTreeCtrlEx::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CTreeCtrl::OnCreate(lpCreateStruct) == -1) {
		return -1;
	}
	
	// Set tree control styles
	long style = GetWindowLong(this->m_hWnd,GWL_STYLE);
	style |= (TVS_HASLINES|TVS_LINESATROOT|TVS_HASBUTTONS|TVS_SHOWSELALWAYS);
	SetWindowLong(this->m_hWnd,GWL_STYLE,style);

	return 0;
}

void CTreeCtrlEx::InitializeCtrl()
{
}

//////////////////////////////////////////////////////////////////////
void CTreeCtrlEx::SetItemFont(HTREEITEM hItem, LOGFONT& logfont)
{
	Color_Font cf;
	if( !m_mapColorFont.Lookup( hItem, cf ) )
		cf.color = (COLORREF)-1;
	cf.logfont = logfont;
	m_mapColorFont[hItem] = cf;
}

//////////////////////////////////////////////////////////////////////
void CTreeCtrlEx::SetItemBold(HTREEITEM hItem, BOOL bBold)
{
	SetItemState(hItem, bBold ? TVIS_BOLD : 0, TVIS_BOLD);
}

//////////////////////////////////////////////////////////////////////
void CTreeCtrlEx::SetItemColor(HTREEITEM hItem, COLORREF color)
{
	Color_Font cf;
	if(!m_mapColorFont.Lookup(hItem, cf))
		cf.logfont.lfFaceName[0] = '\0';
	cf.color = color;
	m_mapColorFont[hItem] = cf;
}

//////////////////////////////////////////////////////////////////////
BOOL CTreeCtrlEx::GetItemFont(HTREEITEM hItem, LOGFONT * plogfont)
{
	Color_Font cf;
	if(!m_mapColorFont.Lookup(hItem, cf))
		return FALSE;
	if(cf.logfont.lfFaceName[0] == '\0') 
		return FALSE;
	*plogfont = cf.logfont;
	return TRUE;

}

//////////////////////////////////////////////////////////////////////
BOOL CTreeCtrlEx::GetItemBold(HTREEITEM hItem)
{
	return GetItemState(hItem, TVIS_BOLD) & TVIS_BOLD;
}

//////////////////////////////////////////////////////////////////////
COLORREF CTreeCtrlEx::GetItemColor(HTREEITEM hItem)
{
	// Returns (COLORREF)-1 if color was not set
	Color_Font cf;
	if(!m_mapColorFont.Lookup(hItem, cf))
		return (COLORREF) - 1;
	return cf.color;

}

//////////////////////////////////////////////////////////////////////
void CTreeCtrlEx::OnPaint() 
{
	CPaintDC dc(this);

	// Create a memory DC compatible with the paint DC
	CDC memDC;
	memDC.CreateCompatibleDC(&dc);

	CRect rcClip, rcClient;
	dc.GetClipBox( &rcClip );
	GetClientRect(&rcClient);

	// Select a compatible bitmap into the memory DC
	CBitmap bitmap;
	bitmap.CreateCompatibleBitmap( &dc, rcClient.Width(), rcClient.Height() );
	memDC.SelectObject( &bitmap );
	
	// Set clip region to be same as that in paint DC
	CRgn rgn;
	rgn.CreateRectRgnIndirect( &rcClip );
	memDC.SelectClipRgn(&rgn);
	rgn.DeleteObject();
	
	// First let the control do its default drawing.
	CWnd::DefWindowProc(WM_PAINT, (WPARAM)memDC.m_hDC, 0);

	HTREEITEM hItem = GetFirstVisibleItem();

	int iItemCount = GetVisibleCount() + 1;
	while(hItem && iItemCount--)
	{		
		CRect rect;

		// Do not meddle with selected items or drop highlighted items
		UINT selflag = TVIS_DROPHILITED | TVIS_SELECTED;
		Color_Font cf;
	
		//if ( !(GetTreeCtrl().GetItemState( hItem, selflag ) & selflag ) 
		//	&& m_mapColorFont.Lookup( hItem, cf ))
		
		if ((GetItemState(hItem, selflag) & selflag) 
			&& ::GetFocus() == m_hWnd)
			;
		else if (m_mapColorFont.Lookup(hItem, cf))
		{
			CFont *pFontDC;
			CFont fontDC;
			LOGFONT logfont;

			if(cf.logfont.lfFaceName[0] != '\0') 
				logfont = cf.logfont;
			else {
				// No font specified, so use window font
				CFont *pFont = GetFont();
				pFont->GetLogFont( &logfont );
			}

			if(GetItemBold(hItem))
				logfont.lfWeight = 700;

			fontDC.CreateFontIndirect(&logfont);
			pFontDC = memDC.SelectObject(&fontDC );

			if(cf.color != (COLORREF) - 1)
				memDC.SetTextColor(cf.color);
			else
				memDC.SetTextColor(GetSysColor(COLOR_WINDOWTEXT));


			CString sItem = GetItemText(hItem);

			GetItemRect(hItem, &rect, TRUE);
			memDC.SetBkColor( GetSysColor(COLOR_WINDOW));
			memDC.TextOut(rect.left + 2, rect.top + 1, sItem);
			
			memDC.SelectObject(pFontDC);
		}
		hItem = GetNextVisibleItem(hItem);
	}


	dc.BitBlt(rcClip.left, rcClip.top, rcClip.Width(), rcClip.Height(), &memDC, 
				rcClip.left, rcClip.top, SRCCOPY);

	memDC.DeleteDC();
}


//////////////////////////////////////////////
//
//void CTreeCtrlEx::SelectRootItem(void)
//{
//	if (m_hRoot) {
//		SelectItem(m_hRoot);
//		m_hItem = m_hRoot;
//		CurrentSubKeyRefresh();
//	}
//}
//
//void CTreeCtrlEx::LoadTreeView(HKEY hKey, CString csFullKey, HTREEITEM hItem)
//{
//	//
//	if (hItem == m_hRoot) {
//		return;
//	}
//
//	if (!m_bRootKeySet) {
//		m_ntReg.SetRootKey(hKey);
//		m_bRootKeySet = TRUE;
//	}
//
//	//
//	if (hItem == m_hHKCR) {
//		csFullKey = m_ntReg.GetRootPathFor(HKCR);
//	}
//	else if (hItem == m_hHKCU) {
//		csFullKey = m_ntReg.GetRootPathFor(HKCU);
//	}
//	else if (hItem == m_hHKLM) {
//		csFullKey = m_ntReg.GetRootPathFor(HKLM);
//	}
//	else if (hItem == m_hHKU) {
//		csFullKey = m_ntReg.GetRootPathFor(HKU);
//	}
//	else if (hItem == m_hHKCC) {
//		csFullKey = m_ntReg.GetRootPathFor(HKCC);
//	}
//
//	//
//	if (m_ntReg.SetKey(csFullKey,FALSE,TRUE)) {
//		//
//		CStringArray csaKeys;
//
//		if (hItem == NULL) {
//			hItem = m_hRoot;
//		}
//
//		BOOL bOk = FALSE;
//		m_ntReg.GetSubKeyList(csaKeys);
//		for (int n=0; n<csaKeys.GetSize(); n++) {
//			//
//			CString csKey(csFullKey);
//			csKey += _T("\\");
//			csKey += csaKeys.GetAt(n);
//
//			bOk = m_ntReg.SetKey(csKey,FALSE,TRUE);
//
//			int nIcon = 5;
//			if (m_ntReg.m_bHidden) { nIcon = 0; }
//
//			HTREEITEM hNewItem = InsertItem(csaKeys.GetAt(n),nIcon,nIcon,hItem);
//			if (bOk && m_ntReg.GetSubKeyCount() > 0) {
//				InsertItem(FAKEDITEM,5,5,hNewItem,TVI_LAST);
//			}
//			SortChildren(hNewItem);
//		}
//	}
//}
//
//void CTreeCtrlEx::OnRclick(NMHDR* pNMHDR, LRESULT* pResult) 
//{
//	CPoint pt;
//	GetCursorPos(&pt);
//
//	CPoint MousePosition = pt;
//	ScreenToClient ( &MousePosition );
//
//	TVHITTESTINFO HitInfo;
//	HitInfo.pt = MousePosition;
//
//	int Mask = TVHT_ONITEM | TVHT_ONITEMRIGHT;
//	
//	m_hItem = HitTest(&HitInfo);
//
//	theApp.m_clsMainFrm->m_bKeyPressed = TRUE;
//	theApp.m_clsMainFrm->m_bValuePressed = FALSE;
//
//	CurrentSubKeyRefresh();
//
//	if (m_hItem != NULL) {
//		//
//		theApp.m_clsMainFrm->m_bKeyPressed = TRUE;
//		theApp.m_clsMainFrm->m_bValuePressed = FALSE;
//
//		CMenu menu; 
//		VERIFY(menu.LoadMenu(IDR_MAINFRAME));
//		CMenu* pPopup = menu.GetSubMenu(1); 
//		ASSERT(pPopup != NULL); 
//		pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, AfxGetMainWnd());
//	}
//	else {
//		theApp.m_clsMainFrm->m_bKeyPressed = FALSE;
//		theApp.m_clsMainFrm->m_bValuePressed = FALSE;
//	}
//
//	*pResult = 0;
//}
//
//void CTreeCtrlEx::OnClick(NMHDR* pNMHDR, LRESULT* pResult) 
//{
//	//CTreeCtrl &tc=GetTreeCtrl();
//
//	CPoint pt;
//	GetCursorPos(&pt);
//
//	//CPoint MousePosition = pt;
//	//tc.ScreenToClient ( &MousePosition );
//
//	//TVHITTESTINFO HitInfo;
//	//HitInfo.pt = MousePosition;
//
//	//int Mask = TVHT_ONITEM | TVHT_ONITEMRIGHT;
//	//
//	//m_hItem = tc.HitTest(&HitInfo);
//
//	//theApp.m_clsMainFrm->m_bKeyPressed = TRUE;
//	//theApp.m_clsMainFrm->m_bValuePressed = FALSE;
//
//	//CurrentSubKeyRefresh();
//
//	*pResult = 0;
//}
//
//void CTreeCtrlEx::CurrentSubKeyRefresh(void)
//{
//	CString csKeyPath = GetTreePath(m_hItem);
//
//	if (csKeyPath != _T("")) {
//		//
//		if (csKeyPath == _T("CNtRegistry")) {
//			theApp.m_clsLV->m_xList.DeleteAllItems();
//			return;
//		}
//
//		CString csRoot = _T("");
//		CString csKey1 = _T("");
//
//		HKEY hKey = m_ntReg.GetRootKey();
//		if (hKey == HKCC) {
//			csRoot = _T("\\Registry\\Machine\\System\\CurrentControlSet\\Hardware Profiles\\Current");
//			m_csKeyPath = _T("HKCC\\");
//		}
//		else if (hKey == HKCR) {
//			csRoot = _T("\\Registry\\Machine\\Software\\Classes");
//			m_csKeyPath = _T("HKCR\\");
//		}
//		else if (hKey == HKLM) {
//			csRoot = _T("\\Registry\\Machine");
//			m_csKeyPath = _T("HKLM\\");
//		}
//		else if (hKey == HKU) {
//			csRoot = _T("\\Registry\\User");
//			m_csKeyPath = _T("HKU\\");
//		}
//		else {
//			csRoot = m_csSID;
//			m_csKeyPath = _T("HKCU\\");
//		}
//
//		CString csKey = csKeyPath.Right((csKeyPath.GetLength() - csRoot.GetLength()) - 1);
//
//		// This means we are at one of the HKEY_... keys
//		//if (csKey == _T("")) {
//		//	theApp.m_clsLV->m_xList.DeleteAllItems();
//		//	return;
//		//}
//
//		//
//		m_csKeyPath += csKey;
//		theApp.m_clsMainFrm->m_wndStatusBar.SetPaneText(1,m_csKeyPath);
//
//		//CString csAcct,csDomain,csDomainAcct;
//		//m_ntReg.GetObjAcctDomainInfo(SE_REGISTRY_KEY,csKeyPath,csAcct,csDomain);
//		//csDomainAcct.Format("%s\\%s",csDomain,csAcct);
//		//theApp.m_clsMainFrm->m_wndStatusBar.SetPaneText(2,csDomainAcct);
//
//		//
//		if (m_ntReg.SetKey(csKeyPath,FALSE,TRUE)) {
//			//
//			ShowValues();
//			//
//			if (IsExpanded(m_hItem) && m_ntReg.m_bHidden) {
//				SetItemImage(m_hItem,0,0);
//			}
//			else if (m_ntReg.m_bHidden) {
//				SetItemImage(m_hItem,0,0);
//			}
//			else if (IsExpanded(m_hItem)) {
//				SetItemImage(m_hItem,6,6);
//			}
//			else {
//				SetItemImage(m_hItem,5,5);
//			}
//		}
//	}
//}
//
//CString CTreeCtrlEx::GetTreePath(HTREEITEM hItem)
//{
//	CString csReturn(GetItemText(hItem));
//
//	if (csReturn == _T("CNtRegistry")) {
//		return csReturn;
//	}
//
//	CString csRoots = _T("");
//	if (csReturn == _T("HKEY_CLASSES_ROOT")) {
//		m_ntReg.SetRootKey(HKCR);
//	}
//	else if (csReturn == _T("HKEY_CURRENT_USER")) {
//		m_ntReg.SetRootKey(HKCU);
//	}
//	else if (csReturn == _T("HKEY_LOCAL_MACHINE")) {
//		m_ntReg.SetRootKey(HKLM);
//	}
//	else if (csReturn == _T("HKEY_USERS")) {
//		m_ntReg.SetRootKey(HKU);
//	}
//	else if (csReturn == _T("HKEY_CURRENT_CONFIG")) {
//		m_ntReg.SetRootKey(HKCC);
//	}
//
//	csRoots = m_ntReg.GetRootPath();
//	if (csReturn.Left(5) == _T("HKEY_")) {
//		return csRoots;
//	}
//
//	// Ok, we have a real path...
//	HTREEITEM hParent = GetParentItem(hItem);
//	for (;;)
//	{
//		CString csParent = GetItemText(hParent);
//		if (csParent.Left(5) == _T("HKEY_")) {
//			csReturn.Insert(0,_T("\\"));
//			csReturn.Insert(0,csRoots);
//			break;
//		}
//		else {
//			csReturn.Insert(0,_T("\\"));
//			csReturn.Insert(0,csParent);
//		}
//		HTREEITEM hNewParent = GetParentItem(hParent);
//		hParent = hNewParent;
//		if (hParent == NULL) {
//			break;
//		}
//	}
//
//	return csReturn;
//}
//
//
//BOOL CTreeCtrlEx::IsExpanded(HTREEITEM hItem)
//{
//	BOOL bResult = FALSE;
//
//	TVITEM dragItem;
//	dragItem.mask=TVIF_HANDLE | TVIF_STATE;
//	dragItem.hItem=hItem;
//	dragItem.pszText=NULL;
//	if (GetItem(&dragItem)) {
//		bResult=dragItem.state & TVIS_EXPANDED;
//	}
//	return bResult;
//}
//
////BOOL CTreeCtrlEx::IsItemExpanded(HTREEITEM hItem)
////{
////	return (BOOL)GetItemState(hItem, TVIS_EXPANDED) & TVIS_EXPANDED;
////}
//
//void CTreeCtrlEx::TraverseTree(CString csKeyPath)
//{
//	HTREEITEM hRoot = GetRootItem();
//	HTREEITEM hItem = NULL;
//
//	CString csSID = _T("");
//	m_ntReg.LookupSID(csSID);
//
//	CString csRootPath = _T("");
//	csRootPath.Format(_T("\\Registry\\User\\%s"), csSID);
//
//	HKEY hKey=NULL;
//	CString csRootKey = _T("");
//	if (csKeyPath.Left(csRootPath.GetLength()) == csRootPath) {
//		hKey = HKEY_CURRENT_USER;
//		hItem = m_hHKCU;
//	}
//	else if (csKeyPath.Left(14) == _T("\\Registry\\User")) {
//		csRootPath = _T("\\Registry\\User");
//		hKey = HKEY_USERS;
//		hItem = m_hHKU;
//	}
//	else if (csKeyPath.Left(68) == _T("\\Registry\\Machine\\System\\CurrentControlSet\\Hardware Profiles\\Current")) {
//		csRootPath = _T("\\Registry\\Machine\\System\\CurrentControlSet\\Hardware Profiles\\Current");
//		hKey = HKEY_CURRENT_CONFIG;
//		hItem = m_hHKCC;
//	}
//	else if (csKeyPath.Left(34) == _T("\\Registry\\Machine\\SOFTWARE\\Classes")) {
//		csRootPath = _T("\\Registry\\Machine\\SOFTWARE\\Classes");
//		hKey = HKEY_CLASSES_ROOT;
//		hItem = m_hHKCR;
//	}
//	else if (csKeyPath.Left(17) == _T("\\Registry\\Machine")) {
//		csRootPath = _T("\\Registry\\Machine");
//		hKey = HKEY_LOCAL_MACHINE;
//		hItem = m_hHKLM;
//	}
//
//	if (csKeyPath == csRootPath) {
//		Expand(hRoot, TVE_EXPAND);
//		EnsureVisible(hRoot);
//		SelectItem(hRoot);
//		return;
//	}
//
//	CString csPath = _T("");
//	if (csKeyPath.GetLength() > csRootPath.GetLength()) {
//		csPath = csKeyPath.Mid(csRootPath.GetLength()+1);
//	}
//	else {
//		csPath = csKeyPath;
//	}
//
//	SetRedraw(FALSE);
//
//	Expand(hRoot, TVE_EXPAND);
//
//	CTokenEx tok;
//	tok.Split(csPath,_T("\\"));
//
//	hItem = OpenSubKeyItem(hItem, csRootPath, tok.m_csaAddIt.GetAt(0));
//	for (int n=0; n<tok.m_csaAddIt.GetSize()-1; n++) {
//		csRootPath += _T("\\");
//		csRootPath += tok.m_csaAddIt.GetAt(n);
//		hItem = OpenSubKeyItem(hItem, csRootPath, tok.m_csaAddIt.GetAt(n+1));
//
//		if (hItem) {
//			EnsureVisible(hItem);
//			SelectItem(hItem);
//			theApp.m_clsMainFrm->OnKeysRefresh();
//		}
//	}
//	SetRedraw(TRUE);
//}
//
//HTREEITEM CTreeCtrlEx::OpenSubKeyItem(HTREEITEM hItem, CString csSubKey, CString csLook4)
//{
//	HTREEITEM hReturnItem = NULL;
//	SetItemImage(hItem,6,6);
//	
//	CString csName = GetItemText(hItem);
//	HTREEITEM hChildItem = GetChildItem(hItem);
//	if (hChildItem) {
//		//
//		CString csChildName = GetItemText(hChildItem);
//		if ( csChildName.CompareNoCase(FAKEDITEM)==0  ) {
//			//
//			// there is 
//			//
//			// remove this child
//			DeleteItem(hChildItem);
//			
//			if (hItem && GetParentItem(hItem))
//			{
//				LockWindowUpdate(); // CWnd member
//
//				SetCursor(LoadCursor(NULL, IDC_WAIT)); // hourglass
//
//				//
//				if (m_ntReg.SetKey(csSubKey,FALSE,TRUE)) {
//					//
//					CStringArray csaKeys;
//
//					BOOL bOk = FALSE;
//					m_ntReg.GetSubKeyList(csaKeys);
//					for (int n=0; n<csaKeys.GetSize(); n++) {
//						//
//						CString csKey(csSubKey);
//						csKey += _T("\\");
//						csKey += csaKeys.GetAt(n);
//
//						bOk = m_ntReg.SetKey(csKey,FALSE,TRUE);
//
//						int nIcon = 5;
//						if (m_ntReg.m_bHidden) { nIcon = 0; }
//
//						HTREEITEM hNewItem = InsertItem(csaKeys.GetAt(n),nIcon,nIcon,hItem);
//						if (csLook4 == csaKeys.GetAt(n)) {
//							hReturnItem = hNewItem;
//						}
//						if (bOk && m_ntReg.GetSubKeyCount() > 0) {
//							InsertItem(FAKEDITEM,5,5,hNewItem,TVI_LAST);
//						}
//					}
//
//					if (hItem) {
//						SortChildren(hItem);
//						Expand(hItem, TVE_EXPAND);
//						SelectItem(hItem);
//						theApp.m_clsMainFrm->OnKeysRefresh();
//					}
//				}
//				SetCursor(LoadCursor (NULL, IDC_ARROW)); // back to normal cursor
//
//				UnlockWindowUpdate();  // CWnd member
//			}
//		}
//		else {
//			//
//			if (csLook4 == csChildName) {
//				hReturnItem = hItem;
//			}
//			else {
//				// The SubKey is already EXPANDED
//				HTREEITEM hSibling = GetNextSiblingItem(hChildItem);
//				// Go through all of siblings.
//				while (hSibling != NULL) {
//					//
//					CString csSibling = GetItemText(hSibling);
//					if (csLook4 == csSibling) {
//						hReturnItem = hSibling;
//						break;
//					}
//					hSibling = GetNextSiblingItem(hSibling);
//				} // WHILE
//			}
//		}
//	}
//
//	return hReturnItem;
//}
//
//void CTreeCtrlEx::ShowValues(void)
//{
//	theApp.m_clsLV->m_xList.DeleteAllItems();
//
//	UINT uiLen=0;
//	UCHAR* pValue = NULL;
//
//	CStringArray csaValues;
//	if (m_ntReg.GetValueList(csaValues))
//	{
//		int i=0;
//		int nImage=0;
//		BOOL bDefaultUsed = FALSE;
//		for (int n=0; n<csaValues.GetSize(); n++)
//		{
//			nImage=8;
//			int nSize = 0;
//			DWORD dw = 0;
//			CString csValueName = csaValues.GetAt(n);
//			DWORD dwValueType = m_ntReg.GetValueInfo(csValueName, nSize);
//			CString csValue = _T("ERR"), csValueType;
//
//			switch(dwValueType)
//			{
//			case REG_SZ:
//				csValueType = _T("REG_SZ");
//				csValue = m_ntReg.ReadString(csValueName,_T("ERR"));
//				nImage=7;
//				break;
//			case REG_EXPAND_SZ:
//				csValueType = _T("REG_EXPAND_SZ");
//				csValue = m_ntReg.ReadString(csValueName,_T("ERR"));
//				nImage=7;
//				break;
//			case REG_MULTI_SZ:
//				csValueType = _T("REG_MULTI_SZ");
//				m_ntReg.ReadMultiString(csValueName,m_csaNewString);
//				csValue = _T("");
//				for (i=0; i<m_csaNewString.GetSize(); i++)
//				{
//					csValue += m_csaNewString.GetAt(i);
//					csValue += _T(" ");
//				}
//				nImage=7;
//				break;
//			case REG_DWORD:
//				csValueType = _T("REG_DWORD");
//				dw = m_ntReg.ReadDword(csValueName,0);
//				csValue.Format(_T("0x%08x (%u)"), dw, dw);
//				break;
//			case REG_DWORD_BIG_ENDIAN:
//				csValueType = _T("REG_DWORD_BIG_ENDIAN");
//				break;
//			case REG_QWORD:
//				csValueType = _T("REG_QWORD");
//				pValue = m_ntReg.ReadBinary(csValueName,uiLen);
//				break;
//			case REG_BINARY:
//				csValueType = _T("REG_BINARY");
//				pValue = m_ntReg.ReadBinary(csValueName,uiLen);
//				break;
//			case REG_LINK:
//				csValueType = _T("REG_LINK");
//				pValue = m_ntReg.ReadBinary(csValueName,uiLen);
//				break;
//			case REG_RESOURCE_LIST:
//				csValueType = _T("REG_RESOURCE_LIST");
//				pValue = m_ntReg.ReadBinary(csValueName,uiLen);
//				break;
//			case REG_RESOURCE_REQUIREMENTS_LIST:
//				csValueType = _T("REG_RESOURCE_REQUIREMENTS_LIST");
//				pValue = m_ntReg.ReadBinary(csValueName,uiLen);
//				break;
//			case REG_FULL_RESOURCE_DESCRIPTOR:
//				csValueType = _T("REG_FULL_RESOURCE_DESCRIPTOR");
//				pValue = m_ntReg.ReadBinary(csValueName,uiLen);
//				break;
//			case REG_NONE:
//				csValueType = _T("REG_NONE");
//				pValue = m_ntReg.ReadBinary(csValueName,uiLen);
//				break;
//			default:
//				csValueType.Format(_T("REG_UNKNOWN (%u)"), dwValueType);
//				csValue = _T("ERR");
//				break;
//			}
//
//
//			if (csValueName == _T(""))
//			{
//				csValueName = _T("(Default)");
//				bDefaultUsed = TRUE;
//			}
//
//			int nItem = theApp.m_clsLV->m_xList.InsertItem(theApp.m_clsLV->m_xList.GetItemCount(),csValueName,RGB(255,0,255),RGB(255,255,255));
//			theApp.m_clsLV->m_xList.SetItemImage(nItem,0,nImage);
//			theApp.m_clsLV->m_xList.SetItemText(nItem,1,csValueType);
//			if (dwValueType == REG_BINARY || dwValueType == REG_QWORD ||
//				dwValueType == REG_LINK ||
//				dwValueType == REG_RESOURCE_LIST ||
//				dwValueType == REG_RESOURCE_REQUIREMENTS_LIST ||
//				dwValueType == REG_FULL_RESOURCE_DESCRIPTOR) {
//				//
//				csValue = _T("");
//				CString csTmp = _T("");
//
//				if (uiLen == 0) {
//					csValue = _T("00 00 00 00");
//				}
//				else {
//					//
//					for (int n=0; n<(int)uiLen; n++) {
//						csTmp.Format("%02x ",pValue[n]);
//						csValue += csTmp;
//					}
//				}
//				csValue.TrimRight(" ");
//			}
//			theApp.m_clsLV->m_xList.SetItemText(nItem,2,csValue);
//		}
//
//		if (!bDefaultUsed)
//		{
//			int nItem = theApp.m_clsLV->m_xList.InsertItem(0,_T("(Default)"));
//			theApp.m_clsLV->m_xList.SetItemImage(nItem,0,7);
//			theApp.m_clsLV->m_xList.SetItemText(nItem,1,_T("REG_SZ"));
//			theApp.m_clsLV->m_xList.SetItemText(nItem,2,_T("(value not set)"));
//		}
//	}
//}
//
//
//void CTreeCtrlEx::OnTvnKeydown(NMHDR *pNMHDR, LRESULT *pResult)
//{
//	LPNMTVKEYDOWN pTVKeyDown = reinterpret_cast<LPNMTVKEYDOWN>(pNMHDR);
//
//	switch (pTVKeyDown->wVKey)	
//	{
//		case VK_F5:
//			theApp.m_clsMainFrm->OnKeysRefresh();
//			break;
//			
//		case VK_DELETE:
//			{
//				if (m_ntReg.GetSubKeyCount() > 0) {
//					theApp.m_clsMainFrm->OnRecursivelyKey();
//				}
//				else {
//					theApp.m_clsMainFrm->OnDeleteKeyValue();
//				}
//			}
//			break;
//
//		//case VK_UP:
//		//case VK_DOWN:
//		//	m_hItem = tc.GetSelectedItem();
//		//	if (m_hItem != tc.GetRootItem()) {
//		//		theApp.m_clsMainFrm->m_bKeyPressed = TRUE;
//		//		theApp.m_clsMainFrm->m_bValuePressed = FALSE;
//		//		CurrentSubKeyRefresh();
//		//	}
//		//	break;
//
//		case VK_RETURN:
//			m_hItem = GetSelectedItem();
//			if (m_hItem != GetRootItem() && m_hItem != NULL) {
//				Expand(m_hItem, TVE_EXPAND);
//			}
//			break;
//
//		//case VK_ESCAPE:
//		//	TreeView_EndEditLabelNow(tc.m_hWnd,TRUE);
//		//	break;
//
//	}
//	*pResult = 0;
//}
//
//void CTreeCtrlEx::OnTvnSelchanged(NMHDR *pNMHDR, LRESULT *pResult)
//{
//	LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);
//	
//	m_hItem = pNMTreeView->itemNew.hItem;
//	if (m_hItem != GetRootItem() && m_hItem != NULL) {
//		theApp.m_clsMainFrm->m_bKeyPressed = TRUE;
//		theApp.m_clsMainFrm->m_bValuePressed = FALSE;
//		CurrentSubKeyRefresh();
//	}
//	*pResult = 0;
//}
//
//void CTreeCtrlEx::OnTvnBeginlabeledit(NMHDR *pNMHDR, LRESULT *pResult)
//{
//	LPNMTVDISPINFO pTVDispInfo = reinterpret_cast<LPNMTVDISPINFO>(pNMHDR);
//
//	m_hBeforeEditItem = pTVDispInfo->item.hItem;
//
//	m_csBeforeEditLable = _T("");
//	m_csFullPathOld = _T("");
//	if (m_hBeforeEditItem != NULL && 
//		m_hBeforeEditItem != GetRootItem() && 
//		GetParentItem(m_hBeforeEditItem) != GetRootItem()) 
//	{
//		m_csBeforeEditLable = GetItemText(m_hBeforeEditItem);
//		m_csFullPathOld = GetTreePath(m_hBeforeEditItem);
//	}
//	else {
//		TreeView_EndEditLabelNow(this->m_hWnd,TRUE);
//	}
//	*pResult = 0;
//}
//
//void CTreeCtrlEx::OnTvnEndlabeledit(NMHDR *pNMHDR, LRESULT *pResult)
//{
//	LPNMTVDISPINFO pTVDispInfo = reinterpret_cast<LPNMTVDISPINFO>(pNMHDR);
//
//	m_hAfterEditItem = pTVDispInfo->item.hItem;
//
//	m_csAfterEditLable = _T("");
//	if (m_hAfterEditItem != NULL && 
//		m_hAfterEditItem != GetRootItem() && 
//		GetParentItem(m_hAfterEditItem) != GetRootItem()) 
//	{
//		if (m_hAfterEditItem == m_hBeforeEditItem) 
//		{
//			m_csAfterEditLable = pTVDispInfo->item.pszText;
//			if (m_csBeforeEditLable.CompareNoCase(m_csAfterEditLable) != 0) 
//			{
//				if (m_ntReg.RenameKey(m_csFullPathOld, m_csAfterEditLable)) {
//					SetItemText(m_hAfterEditItem, m_csAfterEditLable);
//				}
//				else {
//					SetItemText(m_hAfterEditItem, m_csBeforeEditLable);
//				}
//			}
//		}
//	}
//	*pResult = 0;
//}
//
//BOOL CTreeCtrlEx::PreTranslateMessage(MSG* pMsg)
//{
////	if (pMsg->lParam == TVN_ITEMEXPANDING) 
////	if (pMsg->message == WM_NOTIFY) 
//	if (pMsg->wParam == WM_KEYDOWN) 
//	{
//		MessageBox("Key Down...");
////		MessageBox("Notify...");
////		MessageBox("Expanding...");
//	}
//	return CTreeCtrl::PreTranslateMessage(pMsg);
//}
//
//void CTreeCtrlEx::OnTvnItemexpanding(NMHDR *pNMHDR, LRESULT *pResult)
//{
//	LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);
//
//	MessageBox("Expanding...");
//
//	//
//	UINT uiAction = pNMTreeView->action;
//
//	HTREEITEM hItem = pNMTreeView->itemNew.hItem;
//
//	if (hItem == m_hRoot) {
//		*pResult = 0;
//		return;
//	}
//
//	if (uiAction==TVE_EXPAND) // is being expanded
//	{
//		SetItemImage(hItem,6,6);
//		
//		HTREEITEM hChildItem = GetChildItem(hItem);
//		if (hChildItem)
//		{
//			CString csChildName = GetItemText(hChildItem);
//			if ( csChildName.CompareNoCase(FAKEDITEM)==0  ) // there is 
//			{
//				// remove this child
//				DeleteItem(hChildItem);
//				
//				if (hItem && GetParentItem(hItem))
//				{
//
//					LockWindowUpdate(); // CWnd member
//
//					SetCursor(LoadCursor(NULL, IDC_WAIT)); // hourglass
//
//					CPoint pt;
//					GetCursorPos(&pt);
//
//					CPoint MousePosition = pt;
//					ScreenToClient ( &MousePosition );
//
//					TVHITTESTINFO HitInfo;
//					HitInfo.pt = MousePosition;
//
//					m_hItem = HitTest(&HitInfo);
//					CString csKeyPath = GetTreePath(m_hItem);
//
//					CString csKey(csKeyPath);
//					CString csRoot = m_ntReg.GetRootPath();
//					csKey = csKeyPath.Right((csKeyPath.GetLength() - csRoot.GetLength()) - 1);
//
//					HKEY hKey=NULL;
//					CString csKey1;
//					if (csRoot == m_csSID)
//					{
//						csKey1 = _T("HKEY_CURRENT_USER\\");
//						hKey = HKEY_CURRENT_USER;
//					}
//					else if (csRoot == _T("\\Registry\\Machine\\System\\CurrentControlSet\\Hardware Profiles\\Current"))
//					{
//						csKey1 = _T("HKEY_CURRENT_CONFIG\\");
//						hKey = HKEY_CURRENT_CONFIG;
//					}
//					else if (csRoot == _T("\\Registry\\Machine\\Software\\Classes"))
//					{
//						csKey1 = _T("HKEY_CLASSES_ROOT\\");
//						hKey = HKEY_CLASSES_ROOT;
//					}
//					else if (csRoot == _T("\\Registry\\Machine"))
//					{
//						csKey1 = _T("HKEY_LOCAL_MACHINE\\");
//						hKey = HKEY_LOCAL_MACHINE;
//					}
//					else if (csRoot == _T("\\Registry\\User"))
//					{
//						csKey1 = _T("HKEY_USERS\\");
//						hKey = HKEY_USERS;
//					}
//
//					if (csKeyPath != _T(""))
//					{
//						LoadTreeView(hKey, csKeyPath, m_hItem);
//						SortChildren(m_hItem);
//						Expand(m_hItem, TVE_EXPAND);
//						SelectItem(m_hItem);
//					}
//
//					SetCursor(LoadCursor (NULL, IDC_ARROW)); // back to normal cursor
//
//					UnlockWindowUpdate();  // CWnd member
//				}
//			}
//		}
//	}
//	else {
//		SetItemImage(hItem,5,5);
//	}
//
//	// Make sure we have the "Clicked" on path
//	CString csKeyPath = GetTreePath(hItem);
//	m_ntReg.SetKey(csKeyPath,FALSE,TRUE);
//
//	*pResult = 0;
//}
